html,
body {
    margin: 0;
    padding: 0;
    overflow: hidden;
    background-color: #000;
    font-family: sans-serif;
}

svg#mainSvg {
    position: absolute;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
}

.controls {
    position: absolute;
    bottom: 0;
    left: 0;
    padding: 2em;
    border: 3px solid #fff;
    border-radius: 10px;
    font-size: larger;
    color: #fff;
    background-color: #4d4d4d;

    opacity: 0;
}

.controls:hover {
    opacity: 1;
}

.controls>p {
    display: flex;
    align-items: center;
    gap: 12px;
}